# OZBox.rb http://www.est.hi-ho.ne.jp/atamadekkati/ 
# ダイアログボックスを表示し、入力した値の四角柱を指定点に描画する。
# 2009/07/12 ver1.00 作成(プラグインの勉強用に作成)
# 2009/08/02 ver1.01 アンドゥの表示名を「角材を作る」にする。
#
# 使用者の自己責任でご使用下さい。
# 改変してマシなのができたら私にも教えてください。
# 
# googleのHPにあるshapes.rbなら四角柱以外に多角柱とか円すいなどいろいろ描画できます。
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

class OZBox

def initialize
    @ip1 = nil
    @ip = nil
    @valuedata = [38.mm, 89.mm, 200.mm, ""]
end

# The activate method is called by SketchUp when the tool is first selected.
def activate
  @ip1 = Sketchup::InputPoint.new
  @ip = Sketchup::InputPoint.new
  @drawn = false
  Sketchup::set_status_text "四角柱", SB_VCB_LABEL

  self.reset(nil)
end

# Reset the tool back to its initial state
def reset(view)
    self.promptmake
    # Display a prompt on the status bar
    Sketchup::set_status_text("配置する座標をクリックしてください。", SB_PROMPT)
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
end

# 入力欄が毎回でるようにactivateから分ける。
def promptmake
  prompts = ["角材の幅(x)", "角材の奥行き(y)", "角材の高さ(z)", "名前"]
  values = @valuedata
  results = inputbox prompts, values, "入力,負の値は逆方向に描画"
  return if not results # This means that the user canceld the operation
  @valuedata = results
  @boxwidth, @boxdepth, @boxheight, @boxname = @valuedata
end

# deactivate is called when the tool is deactivated because a different tool was selected
def deactivate(view)
  view.invalidate if @drawn
end

# The onMouseMove method is called whenever the user moves the mouse.
def onMouseMove(flags, x, y, view)
  @ip.pick view, x, y
  if( @ip != @ip1 )
    view.invalidate if( @ip.display? or @ip1.display? )
    @ip1.copy! @ip
    view.tooltip = @ip1.tooltip
  end
end

# The onLButtonDOwn method is called when the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
  if( @ip1.valid? )
    self.create_groupbox(@ip1.position, view)
    self.reset(view)
  end
  view.lock_inference
end

# onKeyDown is called when the user presses a key on the keyboard.
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @ip1.valid? )
            view.lock_inference @ip1
        end
    end
end

# onKeyUp is called when the user releases the key
def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end

# Create new geometry when the user has selected point.
def create_groupbox(point1,  view)
    model = Sketchup.active_model
    model.start_operation "角材を作る"
    entities = model.active_entities
    group = entities.add_group
    entities = group.entities
    group.name = @boxname

    pts = []
    pts[0] = point1
    pts[1] = point1 + [@boxwidth, 0, 0]
    pts[2] = point1 + [@boxwidth, @boxdepth, 0]
    pts[3] = point1 + [0, @boxdepth, 0]
    base = entities.add_face pts

    if( base.normal.dot(Z_AXIS) < 0 )
	    base.pushpull -@boxheight
	  else
	    base.pushpull @boxheight
	  end
    
    model.commit_operation
end

end # class 

#=============================================================================
def oz_box_tool
    Sketchup.active_model.select_tool OZBox.new
end

# Add a menu 
if( not file_loaded?("OZBox.rb") )
    add_separator_to_menu("Draw")
    UI.menu("Draw").add_item("角材") { oz_box_tool }
end

#-----------------------------------------------------------------------------
file_loaded("OZBox.rb")
